/*
	FreeRTOS V5.4.1 - Copyright (C) 2009 Real Time Engineers Ltd.

	This file is part of the FreeRTOS distribution.

	FreeRTOS is free software; you can redistribute it and/or modify it	under 
	the terms of the GNU General Public License (version 2) as published by the 
	Free Software Foundation and modified by the FreeRTOS exception.
	**NOTE** The exception to the GPL is included to allow you to distribute a
	combined work that includes FreeRTOS without being obliged to provide the 
	source code for proprietary components outside of the FreeRTOS kernel.  
	Alternative commercial license and support terms are also available upon 
	request.  See the licensing section of http://www.FreeRTOS.org for full 
	license details.

	FreeRTOS is distributed in the hope that it will be useful,	but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along
	with FreeRTOS; if not, write to the Free Software Foundation, Inc., 59
	Temple Place, Suite 330, Boston, MA  02111-1307  USA.


	***************************************************************************
	*                                                                         *
	* Looking for a quick start?  Then check out the FreeRTOS eBook!          *
	* See http://www.FreeRTOS.org/Documentation for details                   *
	*                                                                         *
	***************************************************************************

	1 tab == 4 spaces!

	Please ensure to read the configuration and relevant port sections of the
	online documentation.

	http://www.FreeRTOS.org - Documentation, latest information, license and
	contact details.

	http://www.SafeRTOS.com - A version that is certified for use in safety
	critical systems.

	http://www.OpenRTOS.com - Commercial support, development, porting,
	licensing and training services.
*/

/* Environment includes. */
#include <targets/LPC23xx.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"

/* Demo app includes. */

#include "portlcd.h"
#include "flash.h"


/* Demo application definitions. */
#define mainQUEUE_SIZE						( 3 )
#define mainCHECK_DELAY						( ( portTickType ) 5000 / portTICK_RATE_MS )
#define mainBASIC_WEB_STACK_SIZE            ( configMINIMAL_STACK_SIZE * 2 )

/* Task priorities. */
#define mainQUEUE_POLL_PRIORITY				( tskIDLE_PRIORITY + 2 )
#define mainCHECK_TASK_PRIORITY				( tskIDLE_PRIORITY + 3 )
#define mainFLASH_PRIORITY                  ( tskIDLE_PRIORITY + 2 )



void setRTC(void);
void startRTC(void);
void feedWatchdog(void);
void startWatchdog(void);

/*
 * Checks the status of all the demo tasks then prints a message to the
 * CrossStudio terminal IO windows.  The message will be either PASS or FAIL
 * depending on the status of the demo applications tasks.  A FAIL status will
 * be latched.
 *
 * Messages are not written directly to the terminal, but passed to vPrintTask
 * via a queue.
 */
static void vCheckTask( void *pvParameters );

/* 
 * The task that handles the uIP stack.  All TCP/IP processing is performed in
 * this task.
 */
extern void vuIP_Task( void *pvParameters );

/* 
 * The task that handles the electricity meter handling/calculations.
 */
extern void vMeters_Task( void *pvParameters );

/*
 * The LCD is written two by more than one task so is controlled by a 
 * 'gatekeeper' task.  This is the only task that is actually permitted to 
 * access the LCD directly.  Other tasks wanting to display a message send
 * the message to the gatekeeper.
 */
static void vLCDTask( void *pvParameters );

/* The queue used to send messages to the LCD task. */
xQueueHandle xLCDQueue;

/*-----------------------------------------------------------*/

int main (void)
{

  startWatchdog();
  startRTC(); /* start the RTC */

  /* Setup the led's on the MCB2300 board */
  vParTestInitialise();

  //testUART3();

  /* Create the queue used by the LCD task.  Messages for display on the LCD
  are received via this queue. */
  //xLCDQueue = xQueueCreate( mainQUEUE_SIZE, sizeof( xLCDMessage ) );

  /* Create the lwIP task.  This uses the lwIP RTOS abstraction layer.*/
  xTaskCreate( vuIP_Task, ( signed portCHAR * ) "uIP", mainBASIC_WEB_STACK_SIZE, NULL, mainCHECK_TASK_PRIORITY - 1, NULL );

//  vStartBlockingQueueTasks( mainBLOCK_Q_PRIORITY );
  vStartMetersTask( mainFLASH_PRIORITY );

  vStartSolarTask( mainFLASH_PRIORITY );
	/* Start the standard demo tasks. */
//  vStartBlockingQueueTasks( mainBLOCK_Q_PRIORITY );
//  vCreateBlockTimeTasks();
  vStartLEDFlashTasks( mainFLASH_PRIORITY );
//  vStartSemaphoreTasks( mainSEM_TEST_PRIORITY );
//  vStartPolledQueueTasks( mainQUEUE_POLL_PRIORITY );
//  vStartIntegerMathTasks( mainINTEGER_TASK_PRIORITY );

	/* Start the tasks defined within this file/specific to this demo. */
 //   xTaskCreate( vCheckTask, ( signed portCHAR * ) "Check", configMINIMAL_STACK_SIZE, NULL, mainCHECK_TASK_PRIORITY, NULL );
 

    
    // LCD task still interferes with UART3 (due to PINSEL3 and SCS setting in LCD_init */
  //  xTaskCreate( vLCDTask, ( signed portCHAR * ) "LCD", configMINIMAL_STACK_SIZE, NULL, mainCHECK_TASK_PRIORITY - 1, NULL );

	/* The suicide tasks must be created last as they need to know how many
	tasks were running prior to their creation in order to ascertain whether
	or not the correct/expected number of tasks are running at any given time. */
 //   vCreateSuicidalTasks( mainCREATOR_TASK_PRIORITY );


	/* Start the scheduler. */
	vTaskStartScheduler();

    /* Will only get here if there was insufficient memory to create the idle
    task. */
	return 0; 
}
/*-----------------------------------------------------------*/

static void vCheckTask( void *pvParameters )
{
portBASE_TYPE xErrorOccurred = pdFALSE;
portTickType xLastExecutionTime;
unsigned portBASE_TYPE uxColumn = 0;
xLCDMessage xMessage;

	xLastExecutionTime = xTaskGetTickCount();

	xMessage.xColumn = 0;
	xMessage.pcMessage = "PASS";

    for( ;; )
	{
		/* Perform this check every mainCHECK_DELAY milliseconds. */
		vTaskDelayUntil( &xLastExecutionTime, mainCHECK_DELAY );

		/* Has an error been found in any task? */



        LCD_cls();
        xMessage.xColumn++;
        LCD_gotoxy( ( uxColumn & 0x07 ) + 1, ( uxColumn & 0x01 ) + 1 );

        if( xErrorOccurred == pdTRUE )
        {
            xMessage.pcMessage = "FAIL";
        }

		/* Send the message to the LCD gatekeeper for display. */
		xQueueSend( xLCDQueue, &xMessage, portMAX_DELAY );
	}
}
/*-----------------------------------------------------------*/

void vLCDTask( void *pvParameters )
{
xLCDMessage xMessage;

	/* Initialise the LCD and display a startup message. */
	LCD_init();
	LCD_cur_off();
    LCD_cls();    
    LCD_gotoxy( 1, 1 );
    LCD_puts( ( signed portCHAR * ) "www.FreeRTOS.org" );

	for( ;; )
	{
		/* Wait for a message to arrive that requires displaying. */
		while( xQueueReceive( xLCDQueue, &xMessage, portMAX_DELAY ) != pdPASS );
		
		/* Display the message.  Print each message to a different position. */
		LCD_cls();
		LCD_gotoxy( ( xMessage.xColumn & 0x07 ) + 1, ( xMessage.xColumn & 0x01 ) + 1 );
		LCD_puts( xMessage.pcMessage );
	}

}
/*-----------------------------------------------------------*/

/* Keep the compiler quiet. */
#include <stdio.h>
int __putchar( int c )
{
    return EOF;
}

void setRTC(void)
{
/*
  This function needs to be called when the RTC battery was replaces

  SEC 6 Seconds value in the range of 0 to 59 R/W 0xE002 4020
  MIN 6 Minutes value in the range of 0 to 59 R/W 0xE002 4024
  HOUR 5 Hours value in the range of 0 to 23 R/W 0xE002 4028
  DOM 5 Day of month value in the range of 1 to 28, 29, 30,
  or 31 (depending on the month and whether it is a
  leap year).[1]
  R/W 0xE002 402C
  DOW 3 Day of week value in the range of 0 to 6[1] R/W 0xE002 4030
  DOY 9 Day of year value in the range of 1 to 365 (366 for
  leap years)[1]
  R/W 0xE002 4034
  MONTH 4 Month value in the range of 1 to 12 R/W 0xE002 4038
  YEAR 12 Year value in the range of 0 to 4095 R/W 0xE002 403C
*/
  RTC_SEC = 00;
  RTC_MIN = 22;
  RTC_HOUR = 17;
  RTC_DOM = 11;
  RTC_DOW = 2-1;
  RTC_DOY = 223;
  RTC_MONTH = 8;
  RTC_YEAR = 2009;
}

void startRTC(void)
{
  // Initial RTC Function
  RTC_CCR  = 0;	// Reset All Bit Control
  RTC_CCR |= 0x00000010; // CLKSRC = 1 = Used EXT 32.768 KHz
  RTC_CCR |= 0x00000002; // Reset   Clock
  RTC_CCR &= 0xFFFFFFFD; // Release Reset
  RTC_CCR |= 0x00000001; // Start RTC Clock	
}


void startWatchdog(void)
{
  WDCLKSEL = 2; /* feed Watchdog from RTC */
  WDTC = 32768*10; /* timeout value (gets multiplied x4) */
  WDMOD = 3; /* WDEN and WDRESET: reset microcontroller in case of time-out */

  /* activate watchdog */
  feedWatchdog();

}

void feedWatchdog(void)
{
    portENTER_CRITICAL();
    WDFEED = 0xAA;
    WDFEED = 0x55;
    portEXIT_CRITICAL();
 }